""" set the path to a custom FW config file """

__author__ = 'Ivan Kondov'
__copyright__ = 'Copyright 2020, Karlsruhe Institute of Technology'
__email__ = 'ivan.kondov@kit.edu'
__maintainer__ = 'Ivan Kondov'

import os

curr_dir = os.path.dirname(os.path.abspath(__file__))
os.environ["FW_CONFIG_FILE"] = os.path.join(curr_dir, 'fw_config.yaml')



""" JSON schema validator for FireWorks working with relative paths """

__author__ = 'Ivan Kondov'
__copyright__ = 'Copyright 2020, Karlsruhe Institute of Technology'
__email__ = 'ivan.kondov@kit.edu'
__maintainer__ = 'Ivan Kondov'

import os
import json
from pathlib import Path
import jsonschema

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
SCHEMA_DIR = os.path.join(ROOT_DIR, 'schema')


def validate(instance, schema_name):
    """ JSON schema validator working with relative paths """
    schema_file = schema_name.lower() + '.json'
    schema_path = os.path.join(SCHEMA_DIR, schema_file)
    with open(schema_path, 'rt') as fileh:
        schema_dict = json.load(fileh)
    base_uri = Path(os.path.abspath(schema_path)).as_uri()
    custom_res = jsonschema.RefResolver(base_uri, schema_dict)
    jsonschema.validate(instance, schema_dict, resolver=custom_res,
                        format_checker=jsonschema.FormatChecker())



""" Unit tests for FireWorks JSON schema """

__author__ = 'Ivan Kondov'
__copyright__ = 'Copyright 2020, Karlsruhe Institute of Technology'
__email__ = 'ivan.kondov@kit.edu'
__maintainer__ = 'Ivan Kondov'

import os
import unittest
import json
from jsonschema.exceptions import ValidationError
from jsonschema.exceptions import SchemaError
from jsonschema import Draft7Validator
import fireworks_schema

SAMPLES_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'samples')


class JSONSchemaTest(unittest.TestCase):
    """ validate the FireWorks JSON schema """

    def test_validate_schema(self):
        """ validate the schema against the metaschema """
        schemas = [
            'addfilestask.json',
            'archivedirtask.json',
            'backgroundtask.json',
            'commandlinetask.json',
            'commonadapter.json',
            'compressdirtask.json',
            'deletefilestask.json',
            'dupefinder.json',
            'filedeletetask.json',
            'filetransfertask.json',
            'filewritetask.json',
            'firetask.json',
            'firework.json',
            'foreachtask.json',
            'fwaction.json',
            'fwconfig.json',
            'fworker.json',
            'generic.json',
            'getfilestask.json',
            'importdatatask.json',
            'joindicttask.json',
            'joinlisttask.json',
            'launch.json',
            'launchpad.json',
            'links.json',
            'pytask.json',
            'scripttask.json',
            'spec.json',
            'templatewritertask.json',
            'tracker.json',
            'workflow.json'
        ]
        for schema in schemas:
            schema_file = os.path.join(fireworks_schema.SCHEMA_DIR, schema)
            with open(schema_file, 'r') as fileh:
                schema_dict = json.load(fileh)
            try:
                Draft7Validator.check_schema(schema_dict)
            except SchemaError as err:
                self.fail('Schema validation error: '+err.message)


class JSONSchemaValidatorTest(unittest.TestCase):
    """ test the FireWorks schema validator using sample documents """

    def test_validate(self):
        """ validate a set of samples against the schema directly """
        schemas = ['Firetask', 'Firework', 'Workflow', 'LaunchPad', 'FWorker',
                   'CommonAdapter']
        for schema in schemas:
            path = os.path.join(SAMPLES_DIR, schema.lower())
            for wf_file in os.listdir(path):
                with open(os.path.join(path, wf_file), 'r') as fileh:
                    try:
                        fireworks_schema.validate(json.load(fileh), schema)
                    except ValidationError as err:
                        self.fail('Validation error: '+err.message)

    def test_validate_invalid(self):
        """ validate a set of invalid workflow samples against the schema """
        for schema in ['Workflow', 'FWorker']:
            path = os.path.join(SAMPLES_DIR, schema.lower()+'-invalid')
            for wf_file in os.listdir(path):
                with open(os.path.join(path, wf_file), 'r') as fileh:
                    inst = json.load(fileh)
                    with self.assertRaises(ValidationError):
                        fireworks_schema.validate(inst, schema)


if __name__ == '__main__':
    unittest.main()



""" fw_serialisers tests with JSON schema validation """

__author__ = 'Ivan Kondov'
__copyright__ = 'Copyright 2020, Karlsruhe Institute of Technology'
__email__ = 'ivan.kondov@kit.edu'
__maintainer__ = 'Ivan Kondov'

import os
import unittest
from jsonschema.exceptions import ValidationError
import fw_config_path
from fireworks.fw_config import JSON_SCHEMA_VALIDATE
from fireworks.utilities.fw_serializers import load_object_from_file

SAMPLES_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'samples')


class JSONSchemaFWserialisersTest(unittest.TestCase):
    """ test validators in fw_serializers.py, requires JSON_SCHEMA_VALIDATE """

    # skip due to runtime errors after validation (with status Error)
    skip_list = [
        'customtask.json',
        'lambda_task.json',
        'launchpad_ssl_x509.json',
        'launchpad_ssl.json',
        'file_transfer_task_2.json',
        'file_transfer_task.json',
        'filetransfertask1.json'
    ]

    def setUp(self):
        if not JSON_SCHEMA_VALIDATE:
            raise unittest.SkipTest('Skipping because JSON_SCHEMA_VALIDATE=False')

    def test_validate_load_object_from_file(self):
        """ validate Firetask, CommonAdapter schemas via load_object_from_file() """
        schemas = ['Firetask', 'CommonAdapter']
        for schema in schemas:
            path = os.path.join(SAMPLES_DIR, schema.lower())
            for sfile in os.listdir(path):
                if sfile in self.skip_list:
                    continue
                try:
                    load_object_from_file(os.path.join(path, sfile))
                except ValidationError as err:
                    self.fail('Validation error: '+err.message)

    def _validate_from_file(self, module, classname):
        """ validate a set of samples against the schema via from_file() """
        namespace = __import__(module.split('.')[0])
        for subpackage in module.split('.')[1:]:
            namespace = getattr(namespace, subpackage)
        cls = getattr(namespace, classname)
        path = os.path.join(SAMPLES_DIR, classname.lower())
        for sfile in os.listdir(path):
            if sfile in self.skip_list:
                continue
            try:
                cls.from_file(os.path.join(path, sfile))
            except ValidationError as err:
                self.fail('Validation error: '+err.message)

    def test_workflow_from_file(self):
        """ validate Workflow schema via FWSerializable.from_file() """
        self._validate_from_file('fireworks', 'Workflow')

    def test_firework_from_file(self):
        """ validate Firework schema via FWSerializable.from_file() """
        self._validate_from_file('fireworks', 'Firework')

    def test_fworker_from_file(self):
        """ validate FWorker schema via FWSerializable.from_file() """
        self._validate_from_file('fireworks', 'FWorker')

    def test_qadapter_from_file(self):
        """ validate CommonAdapter schema via FWSerializable.from_file() """
        modulename = 'fireworks.user_objects.queue_adapters.common_adapter'
        self._validate_from_file(modulename, 'CommonAdapter')

    def test_lpad_from_file(self):
        """ validate LaunchPad schema via FWSerializable.from_file() """
        self._validate_from_file('fireworks', 'LaunchPad')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

__author__ = 'Ivan Kondov'
__copyright__ = 'Copyright 2020-2021, Karlsruhe Institute of Technology'
__email__ = 'ivan.kondov@kit.edu'
__maintainer__ = 'Ivan Kondov'

import os
import sys
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='fireworks_schema',
    version='1.0.2',
    description='JSON Schema for FireWorks',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Ivan Kondov',
    author_email='ivan.kondov@kit.edu',
    url='https://github.com/ikondov/fireworks_schema',
    keywords=['workflow system', 'json schema', 'fireworks'],
    license='BSD-3-Clause License',
    packages=find_packages(),
    package_data={'fireworks_schema': ['schema/*.json'],
                  'fireworks_schema.tests': ['*.yaml', 'samples/*/*.json']},
    install_requires=['jsonschema>=3.2.0', 'fireworks>=1.9.7'],
    extras_require={},
    classifiers=['Programming Language :: Python',
                 'Development Status :: 4 - Beta',
                 'Intended Audience :: Science/Research',
                 'Intended Audience :: System Administrators',
                 'Intended Audience :: Information Technology',
                 'Operating System :: OS Independent',
                 'Topic :: Other/Nonlisted Topic',
                 'Topic :: Scientific/Engineering'],
    test_suite='nose.collector',
    tests_require=['nose'],
    python_requires='>=3.6',
)
